.gedit-side-panel-paned.pane-separator:dir(ltr),
.gedit-side-panel-paned.pane-separator:hover:dir(ltr) {
    border-radius: 0;
    border-width: 0 1px 0 0;
}

.gedit-side-panel-paned.pane-separator:dir(rtl),
.gedit-side-panel-paned.pane-separator:hover:dir(rtl) {
    border-radius: 0;
    border-width: 0 0 0 1px;
}

.gedit-menu-stack-switcher {
    padding: 12px;
}

statusbar frame {
    border: none;
    padding-left: 6px;
    padding-right: 6px;
}

statusbar button.flat {
    border-radius: 0;
    border-bottom: none;
}

GeditFileBrowserWidget .small-button {
  padding: 2px 4px;
}

